import React, { Fragment } from 'react';
import PropTypes from 'prop-types';

function cc(classes) {
  var out = "";
  var type = typeof classes;

  if (type === "string" || type === "number") {
    return classes || "";
  }

  if (Array.isArray(classes) && classes.length > 0) {
    for (var i = 0, len = classes.length; i < len; i++) {
      var next = cc(classes[i]);
      if (next) {
        out += (out && " ") + next;
      }
    }
  } else {
    for (var key in classes) {
      if (classes.hasOwnProperty(key) && classes[key]) {
        out += (out && " ") + key;
      }
    }
  }

  return out;
}

// https://github.com/lodash/lodash/blob/c1f805f4972843b675056b2786f1165f7db81737/.internal/stringToPath.js

const charCodeOfDot = '.'.charCodeAt(0);
const reEscapeChar = /\\(\\)?/g;
const rePropName = RegExp(
// Match anything that isn't a dot or bracket.
'[^.[\\]]+' + '|' +
// Or match property names within brackets.
'\\[(?:' +
// Match a non-string expression.
'([^"\'].*)' + '|' +
// Or match strings (supports escaping characters).
'(["\'])((?:(?!\\2)[^\\\\]|\\\\.)*?)\\2' + ')\\]' + '|' +
// Or match "" as the space between consecutive dots or empty brackets.
'(?=(?:\\.|\\[\\])(?:\\.|\\[\\]|$))', 'g');

/**
 * Converts `string` to a property path array.
 *
 * @private
 * @param {string} string The string to convert.
 * @returns {Array} Returns the property path array.
 */
function stringToPath(string) {
    const result = [];
    if (string.charCodeAt(0) === charCodeOfDot) {
        result.push('');
    }
    string.replace(rePropName, (match, expression, quote, subString) => {
        let key = match;
        if (quote) {
            key = subString.replace(reEscapeChar, '$1');
        } else if (expression) {
            key = expression.trim();
        }
        result.push(key);
    });
    return result;
}

function isFunction(obj) {
    return typeof obj === 'function';
}

function isString(obj) {
    return typeof obj === 'string' || obj instanceof String;
}























/**
 * Returns true if a value is null, undefined, or NaN.
 */


function isObject(obj) {
    return obj != null && typeof obj === 'object';
}

function isPlainObject(obj) {
    return isObject(obj) && (obj.constructor === Object // obj = {}
    || obj.constructor === undefined // obj = Object.create(null)
    );
}

function render(component, props) {
    return isFunction(component) ? React.createElement(component, props) : component;
}

var _extends = Object.assign || function (target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i];

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        target[key] = source[key];
      }
    }
  }

  return target;
};













var objectWithoutProperties = function (obj, keys) {
  var target = {};

  for (var i in obj) {
    if (keys.indexOf(i) >= 0) continue;
    if (!Object.prototype.hasOwnProperty.call(obj, i)) continue;
    target[i] = obj[i];
  }

  return target;
};

function getValue(obj, path, def) {
    if (!obj) return def;
    if (!Array.isArray(path)) {
        path = stringToPath(path);
    }
    let ret = obj;

    for (let key of path) {
        if (ret == null || !Object.hasOwnProperty.call(ret, key)) {
            return def;
        }
        ret = ret[key];
    }

    return ret;
}

function call(fn, ...args) {
    return typeof fn === 'function' ? fn.call(this, ...args) : fn;
}

function debounce(fn, ms) {
    let timer;
    return function debounced(...args) {
        clearTimeout(timer);
        timer = setTimeout(() => fn.call(this, ...args), ms);
    };
}

const __map = {};

function defaultsDeep(options, defaults$$1) {
    for (let key of Object.keys(defaults$$1)) {
        if (options[key] !== undefined) {
            if (isPlainObject(options[key]) && isPlainObject(defaults$$1[key]) && defaults$$1[key] !== __map) {
                defaults$$1[key] = defaultsDeep(options[key], defaults$$1[key]);
            } else {
                defaults$$1[key] = options[key];
            }
        }
    }
    return defaults$$1;
}

function deepMerge(a, b) {
    const out = _extends({}, a);
    if (isFunction(b)) {
        b = b(a);
    }
    for (let k of Object.keys(b)) {
        if (isFunction(b[k])) {
            out[k] = b[k](a[k]);
        } else if (isPlainObject(a[k]) && isPlainObject(b[k])) {
            out[k] = deepMerge(a[k], b[k]);
        } else {
            out[k] = b[k];
        }
    }
    return out;
}

function mergeState(newState) {
    return oldState => deepMerge(oldState, newState);
}

function range(start, end, step = 1) {
    if (step === 0) {
        throw new Error("`step` cannot be 0");
    }
    if (end === undefined) {
        end = start - 1;
        start = 0;
    }
    if (step > 0) {
        if (end < start) {
            throw new Error("`end` cannot be less than `start` when `step` > 0");
        }
    } else {
        if (start < end) {
            throw new Error("`start` cannot be less than `end` when `step` < 0");
        }
    }
    const length = Math.floor((end - start) / step + 1);
    return Array.from({ length }, (_, i) => i * step + start);
}

function clamp(nbr, min, max) {
    if (nbr < min) return min;
    if (nbr > max) return max;
    return nbr;
}

/**
 * Removes an index from an array without mutating the original array.
 *
 * @param {Array} array Array to remove value from
 * @param {Number} index Index to remove
 * @param {Number} count
 * @param {Array} replaceWith
 * @returns {Array} Array with `value` removed
 */
function arraySplice(array, index, count = 1, replaceWith = []) {
    if (index < array.length) {
        let copy = [...array];
        copy.splice(index, count, ...replaceWith);
        return copy;
    }
    return array;
}

function pick(obj, keys) {
    const out = Object.create(null);
    if (Array.isArray(keys)) {
        for (let k of keys) {
            if (Object.hasOwnProperty.call(obj, k)) {
                out[k] = obj[k];
            }
        }
    } else {
        for (let k of Object.keys(keys)) {
            if (obj[k] === undefined) {
                out[k] = keys[k];
            } else {
                out[k] = obj[k];
            }
        }
    }
    return out;
}

function ActionLink(_ref) {
    let { onClick } = _ref,
        props = objectWithoutProperties(_ref, ['onClick']);

    if (onClick) {
        props.onClick = ev => {
            ev.preventDefault();
            onClick(ev);
        };
    } else {
        props.onClick = ev => {
            ev.preventDefault();
        };
    }
    return React.createElement('a', _extends({ href: 'javascript:void 0' }, props));
}

ActionLink.propTypes = {
    onClick: PropTypes.func.isRequired
};

const ASC = 'asc';
const DESC = 'desc';
const PAGE_LINKS = 7; // todo: make this a prop/option?

class PureDataTable extends React.Component {

    constructor(props) {
        super(props);
        this._ajaxCounter = 0;

        this._lengthMenu = () => React.createElement(
            'select',
            { value: this.state.length, onChange: this.changeLength, className: this.props.theme.lengthSelect },
            this.props.lengthMenu.map(len => React.createElement(
                'option',
                { key: len, value: len },
                len
            ))
        );

        this._searchInput = () => React.createElement('input', { type: 'search', value: this.state.search.value, onChange: this.changeSearch, className: this.props.theme.searchInput });

        this.changeLength = ev => {
            this._setLength(parseInt(ev.target.value) || this.props.length);
        };

        this.changeSearch = ev => {
            this._refreshState({
                start: 0,
                search: {
                    value: ev.target.value
                }
            }, true);
        };

        this._setLength = length => {
            let start = Math.floor(this.state.start / length) * length;
            this._refreshState({ length, start });
        };

        this.handleLengthWheel = ev => {
            ev.preventDefault();
            let ticks = wheelTicks(ev);
            let idx = this.props.lengthMenu.indexOf(this.state.length);
            let next = idx + ticks;
            if (next >= 0 && next < this.props.lengthMenu.length) {
                this._setLength(this.props.lengthMenu[next]);
            }
        };

        this._refreshState = (partialState, deb) => {
            this[deb ? '_refresh' : '_refreshNow'](partialState);
            this.setState(mergeState(partialState));
        };

        this.draw = (paging = true) => {
            // https://datatables.net/reference/api/draw()
            if (paging === 'full-reset' || paging === true) {
                this._refreshState({ start: 0, search: { value: '' } }, false);
            } else if (paging === 'full-hold' || paging === false) {
                // FIXME: what is the difference between "full-hold" and "page"??
                this._refreshNow();
            } else if (paging === 'page') {
                this._refreshNow();
            }
        };

        this.handlePageWheel = ev => {
            ev.preventDefault();
            this._incPage(wheelTicks(ev))(ev);
        };

        this._incPage = amount => ev => {
            this._setPage(this.currentPage + amount)(ev);
        };

        this._setPage = pg => ev => {
            // ev.preventDefault();
            pg = clamp(pg, 0, this.pageCount - 1);
            if (pg === this.currentPage) return;
            this._refreshState(({ length }) => ({
                start: pg * length
            }));
        };

        this._sortColumn = n => ev => {
            // ev.preventDefault();
            if (!this._isOrderable(n)) return false;
            if (n < 0 || n >= this.props.columns.length) return false;
            const multiSort = ev.shiftKey || ev.ctrlKey;
            if (multiSort) {
                let idx = this.state.order.findIndex(o => o[0] === n);
                if (idx < 0) {
                    this._refreshState({
                        order: order => [...order, [n, ASC]]
                    });
                } else {
                    this._refreshState({
                        order: order => arraySplice(order, idx, 1, order[idx][1] === ASC ? [[n, DESC]] : [])
                    });
                }
            } else {
                let dir = this.state.order.length && this.state.order[0][0] === n && this.state.order[0][1] === ASC ? DESC : ASC;
                this._refreshState({
                    start: 0,
                    order: [[n, dir]]
                });
            }
        };

        this._refresh = debounce(this._refreshNow, this.props.searchDelay);
        this.state = _extends({}, pick(props, ['start', 'length', 'search', 'order']), {
            recordsTotal: null,
            recordsFiltered: null,
            data: [],
            loading: true,
            error: null
        });
        this._refs = {};
    }

    componentDidMount() {
        this._refreshNow();
    }

    async _refreshNow(partialState) {
        let state = this.state;
        if (partialState) {
            state = deepMerge(state, partialState);
        }
        if (isFunction(this.props.data)) {
            this.setState({ loading: true });
            // https://datatables.net/manual/server-side
            let resp = await this.props.data({
                draw: ++this._ajaxCounter,
                start: state.start,
                length: state.length,
                search: state.search,
                order: state.order.map(([column, dir]) => ({ column, dir })),
                columns: this.props.columns.map(c => pick(c, {
                    data: undefined,
                    name: undefined,
                    orderable: true,
                    search: {
                        value: '',
                        regex: false
                    },
                    searchable: true
                }))
            });
            if (resp.draw < this._ajaxCounter) return;
            // TODO: if resp.data.length === resp.recordsTotal, switch to client-side filtering??
            // TODO: if server over-fetches (returns enough for page 2) allow going to next page without re-fetching?
            const data = resp.data.slice(0, state.length);
            this.setState({
                data,
                loading: false,
                recordsTotal: resp.recordsTotal,
                recordsFiltered: resp.recordsFiltered,
                error: resp.error
            });
        } else if (Array.isArray(this.props.data)) {
            // TODO: pre-process the data; add a `key` and an integer version of number columns...
            let filteredData = this.props.data;
            if (state.search.value) {
                if (state.search.regex) {
                    try {
                        const re = new RegExp(state.search.value, 'i');
                        filteredData = filteredData.filter(row => this.props.columns.some((_, n) => {
                            return re.test(this._getValue(row, n, 'filter'));
                        }));
                    } catch (_) {}
                } else {
                    const searchTerms = state.search.value.trim().toLocaleLowerCase().split(/\s+/);
                    filteredData = filteredData.filter(row => {
                        const values = this.props.columns.map((_, n) => this._getValue(row, n, 'filter').toLocaleLowerCase());
                        return searchTerms.every(term => {
                            return this.props.columns.some((_, n) => {
                                return values[n].includes(term);
                            });
                        });
                    });
                }
            }
            this.setState({
                data: filteredData.slice(state.start, state.start + state.length),
                recordsTotal: this.props.data.length,
                recordsFiltered: filteredData.length,
                loading: false
            });
        }
    }

    // static async getDerivedStateFromProps(nextProps, prevState) {
    //     const data = await nextProps.data; // fixme: i don't think we're allowed to await.
    //     if(Array.isArray(data)) {
    //         if(!nextProps.data.length) {
    //             return {normData: []};
    //         }
    //         // const cols = Object.keys(nextProps.data[0]);
    //         const cols = nextProps.columns.map((c,i) => c.data === undefined ? i : data).filter(x => x !== null);
    //         const normData = data.map(row => {
    //             return cols.map(c => row[c].toLocaleLowerCase())
    //         })
    //         return {normData};
    //     }
    //     return null;
    // }

    get currentPage() {
        return Math.floor(this.state.start / this.state.length);
    }

    get pageCount() {
        return this.state.recordsFiltered == null ? 0 : Math.ceil(this.state.recordsFiltered / this.state.length);
    }

    _getValue(row, colIdx, type) {
        // https://datatables.net/reference/option/columns.render
        // TODO: support `type` and orthogonal data https://datatables.net/examples/ajax/orthogonal-data.html
        const col = this.props.columns[colIdx];
        if (col.data) {
            return getValue(row, col.data);
        } else {
            return row[colIdx];
        }
    }

    _isOrderable(n) {
        const col = this.props.columns[n];
        return col.orderable === undefined || col.orderable;
    }

    // TODO: swipe right/left events?? assuming there's no horizontal scrolling

    render() {
        const { theme, columns, language, columnKey, rowKey, lengthMenu, className, rowComponent, cellComponent } = this.props;
        const { data, loading, recordsFiltered, recordsTotal, start, length, search, order } = this.state;
        const { currentPage, pageCount } = this;

        const sortIdxMap = columns.map((col, n) => order.findIndex(o => o[0] === n));
        const sortDirMap = sortIdxMap.map(idx => idx < 0 ? null : order[idx][1]);
        // console.log(sortDirMap);
        const sortClassMap = {
            [ASC]: theme.sortAsc,
            [DESC]: theme.sortDesc
        };

        const extraProps = { theme };

        return React.createElement(
            'div',
            { className: cc([theme.wrapper, className, loading ? theme.loading : false]) },
            React.createElement(
                'div',
                { className: cc([theme.controlBar, theme.searchBar]) },
                language.lengthMenu && lengthMenu && lengthMenu.length ? React.createElement(
                    'div',
                    { className: cc([theme.length]), onWheel: this.handleLengthWheel },
                    render(language.lengthMenu, _extends({ Menu: this._lengthMenu }, extraProps))
                ) : null,
                React.createElement(
                    'div',
                    { className: cc(theme.search) },
                    language.search ? render(language.search, _extends({ Input: this._searchInput }, extraProps)) : null
                ),
                search.value ? React.createElement(
                    'div',
                    { className: cc(theme.searchText) },
                    'Search results for \u201C',
                    search.value,
                    '\u201D'
                ) : null
            ),
            React.createElement(
                'table',
                { role: 'grid', className: cc(theme.table) },
                React.createElement(
                    'thead',
                    { className: cc(theme.thead) },
                    React.createElement(
                        'tr',
                        { role: 'row', className: cc([theme.tr, theme.hrow]) },
                        columns.map((col, n) => {
                            let title = React.createElement(
                                'span',
                                { className: cc(theme.title) },
                                call(col.title)
                            );
                            const sortDir = sortDirMap[n];
                            const orderable = !!this._isOrderable(n);
                            let sortClass;
                            if (sortDirMap[n]) {
                                sortClass = [theme.sorted, sortClassMap[sortDirMap[n]]];
                            } else if (orderable) {
                                sortClass = theme.unsorted;
                            }
                            if (orderable) {
                                title = React.createElement(
                                    ActionLink,
                                    { className: cc(theme.titleWrap), onClick: this._sortColumn(n) },
                                    title,
                                    orderable && language.sortIcons ? !sortDir ? language.sortIcons.unsorted : sortDir === ASC ? language.sortIcons.ascending : language.sortIcons.descending : null
                                );
                            } else {
                                title = React.createElement(
                                    'span',
                                    { className: cc(theme.titleWrap) },
                                    title
                                );
                            }

                            const { width, minWidth, maxWidth } = col;

                            // let sortDir = orderIdx < 0 ? null : order[orderIdx][1];
                            // console.log(orderIdx);
                            return React.createElement(
                                'th',
                                {
                                    key: columnKey(col, n),
                                    className: cc([theme.cell, theme.th, col.className, orderable ? theme.orderable : theme.unorderable, sortClass]),
                                    scope: 'col',
                                    role: 'columnheader',
                                    'aria-sort': sortDirMap[n] ? sortDirMap[n] === ASC ? 'ascending' : 'descending' : 'none',
                                    style: { width, minWidth, maxWidth }
                                },
                                title
                            );
                        })
                    )
                ),
                React.createElement(
                    'tbody',
                    null,
                    data.length ? data.map((row, m) => {
                        const cells = columns.map((col, n) => {

                            // https://datatables.net/reference/option/columns.render


                            // console.log(row,col,m,n);
                            // let value;
                            // if(Array.isArray(row)) {
                            //     value 
                            // }
                            let value = this._getValue(row, n, 'display');
                            let cell;
                            if (col.render) {
                                cell = render(col.render, _extends({
                                    data: value,
                                    type: 'display',
                                    row,
                                    meta: {
                                        row: m,
                                        col: n
                                    }
                                }, extraProps));
                            } else {
                                cell = value;
                            }

                            return {
                                key: columnKey(col, n),
                                attrs: {
                                    className: cc([theme.cell, theme.td, col.className, sortDirMap[n] ? [sortClassMap[sortDirMap[n]], theme.sorted, theme[`sort${sortIdxMap[n] + 1}`]] : theme.unsorted]),
                                    children: cell
                                },
                                index: n,
                                data: value,
                                row: row
                            };
                        });

                        return render(rowComponent, {
                            key: rowKey(row, m),
                            attrs: {
                                role: "row",
                                className: cc([theme.tr, theme.drow, m % 2 === 0 ? theme.even : theme.odd]),
                                children: cells.map(c => render(cellComponent, c))
                            },
                            data: row,
                            index: m
                        });

                        // TODO: some kind of "End of list" padding so that the final page isn't shorter than the rest?
                    }) :
                    // TODO: some kind of filler cell component while the data is loading, so we have some lorem to put behind the loading splash?
                    loading ? React.createElement(
                        'tr',
                        { className: [theme.tr] },
                        React.createElement(
                            'td',
                            { className: cc([theme.td, theme.loadingRecords, theme.empty]), colSpan: columns.length },
                            language.loadingRecords
                        )
                    ) : recordsTotal > 0 ? React.createElement(
                        'tr',
                        { className: [theme.tr] },
                        React.createElement(
                            'td',
                            { className: cc([theme.td, theme.zeroRecords, theme.empty]), colSpan: columns.length },
                            language.zeroRecords
                        )
                    ) : React.createElement(
                        'tr',
                        { className: [theme.tr] },
                        React.createElement(
                            'td',
                            { className: cc([theme.td, theme.noData, theme.empty]), colSpan: columns.length },
                            language.emptyTable
                        )
                    )
                )
            ),
            React.createElement(
                'div',
                { className: cc([theme.controlBar, theme.infoBar]) },
                language.info ? React.createElement(
                    'div',
                    { className: cc(theme.pageInfo) },
                    !recordsFiltered ? loading ? render(language.infoLoading, extraProps) : render(language.infoEmpty, extraProps) : render(language.info, _extends({
                        start: (start + 1).toLocaleString(),
                        end: Math.min(start + data.length, recordsFiltered).toLocaleString(),
                        total: recordsFiltered.toLocaleString(),
                        max: recordsTotal.toLocaleString(),
                        length: length.toLocaleString()
                    }, extraProps))
                ) : null,
                React.createElement(
                    'div',
                    { className: cc(theme.pagination), onWheel: this.handlePageWheel },
                    currentPage <= 0 ? React.createElement(
                        'span',
                        { className: cc([theme.button, theme.disabled]) },
                        'Previous'
                    ) : React.createElement(
                        ActionLink,
                        { className: cc(theme.button), onClick: this._incPage(-1) },
                        'Previous'
                    ),
                    !recordsFiltered ? loading ? React.createElement(
                        'span',
                        { className: cc(theme.pageNumberSpacer) },
                        '\u22EF'
                    ) : React.createElement(
                        'span',
                        { className: cc(theme.button) },
                        '\u2013'
                    ) : React.createElement(PageNumbers, _extends({}, extraProps, { currentPage: currentPage, pageCount: pageCount, setPage: this._setPage })),
                    currentPage >= pageCount - 1 ? React.createElement(
                        'span',
                        { className: cc([theme.button, theme.disabled]) },
                        'Next'
                    ) : React.createElement(
                        ActionLink,
                        { className: cc(theme.button), onClick: this._incPage(1) },
                        'Next'
                    )
                ),
                React.createElement(
                    'span',
                    { className: cc(theme.pageXofY) },
                    'Page ',
                    currentPage + 1,
                    pageCount ? React.createElement(
                        Fragment,
                        null,
                        ' of ',
                        pageCount
                    ) : null
                )
            ),
            loading && language.processing ? render(language.processing, extraProps) : null
        );
    }
}

function PageNumbers({ currentPage, pageCount, theme, setPage }) {
    let pageNumbers;
    const lastPage = pageCount - 1;
    if (pageCount <= PAGE_LINKS) {
        pageNumbers = pageCount ? range(pageCount) : [];
    } else if (currentPage < 4) {
        pageNumbers = [0, 1, 2, 3, 4, pageCount - 1];
    } else if (currentPage > pageCount - 5) {
        pageNumbers = [0, lastPage - 4, lastPage - 3, lastPage - 2, lastPage - 1, lastPage];
    } else {
        pageNumbers = [0, currentPage - 1, currentPage, currentPage + 1, lastPage];
    }

    const spacer = React.createElement(
        'span',
        { className: cc(theme.pageNumberSpacer) },
        '\u22EF'
    );
    let prevPage = -1;

    return pageNumbers.map(pg => {

        let ret = pg === currentPage ? React.createElement(
            'span',
            { key: pg, className: cc([theme.button, theme.current]) },
            pg + 1
        ) : React.createElement(
            ActionLink,
            { key: pg, className: cc([theme.button]), onClick: setPage(pg) },
            pg + 1
        );

        if (pg > prevPage + 1) {
            ret = React.createElement(
                Fragment,
                { key: pg },
                spacer,
                ret
            );
        }

        prevPage = pg;

        return ret;
    });
}

const funcOrNode = PropTypes.oneOfType([PropTypes.func, PropTypes.node]);

PureDataTable.propTypes = {
    theme: PropTypes.object,
    data: PropTypes.oneOfType([PropTypes.func, PropTypes.array]),
    columns: PropTypes.arrayOf(PropTypes.oneOfType([PropTypes.array, PropTypes.shape({
        data: PropTypes.string,
        render: PropTypes.oneOfType([PropTypes.func]
        // PropTypes.shape({
        //     _: PropTypes.func,
        //     filter: PropTypes.func,
        //     display: PropTypes.func,
        // })
        )
    })])),
    language: PropTypes.shape({
        lengthMenu: funcOrNode,
        search: funcOrNode,
        info: funcOrNode,
        infoLoading: funcOrNode,
        infoEmpty: funcOrNode,
        loadingRecords: funcOrNode,

        // Text shown inside the table records when the is no information to be displayed after filtering.
        zeroRecords: funcOrNode,

        // This string is shown in preference to language.zeroRecords when the table is empty of data (regardless of filtering) - i.e. there are zero records in the table.
        emptyTable: funcOrNode,

        sortIcons: PropTypes.shape({
            ascending: funcOrNode,
            descending: funcOrNode,
            unsorted: funcOrNode
        }),

        processing: funcOrNode
    }),
    // https://www.apollographql.com/docs/react/advanced/caching.html#normalization
    rowKey: PropTypes.func,
    columnKey: PropTypes.func,
    lengthMenu: PropTypes.arrayOf(PropTypes.number),
    // https://datatables.net/reference/option/searchDelay
    searchDelay: PropTypes.number,

    // TODO: put page number and maybe search term in https://developer.mozilla.org/en-US/docs/Web/API/Window/sessionStorage
    // put length, sorting, and show/hide column preferences in localStorage.
    // add reset button?
    storageKey: PropTypes.string,
    rowComponent: PropTypes.func,
    cellComponent: PropTypes.func
};

function DataTableRow({ attrs }) {
    return React.createElement('tr', attrs);
}

function DataTableCell({ attrs }) {
    return React.createElement('td', attrs);
}

class DataTable extends React.Component {
    constructor(...args) {
        var _temp;

        return _temp = super(...args), this._ref = React.createRef(), _temp;
    }

    componentDidMount() {
        if (this.props.api) {
            this.props.api({
                draw: (...args) => this._ref.current.draw(...args)
            });
        }
    }

    componentWillUnmount() {
        if (this.props.api) {
            this.props.api(null);
        }
    }

    render() {
        const _defaultsDeep = defaultsDeep(this.props, {
            className: undefined,
            theme: __map,
            data: [],
            rowKey: (row, idx) => row._id || row.id || row._key || row.key || idx,
            columnKey: (col, idx) => col._id || col.id || col._key || col.key || col.name || idx,
            language: {
                // https://datatables.net/reference/option/language
                lengthMenu: ({ Menu }) => React.createElement(
                    'label',
                    null,
                    'Show ',
                    React.createElement(Menu, null),
                    ' entries'
                ),
                search: ({ Input }) => React.createElement(
                    'label',
                    null,
                    React.createElement(
                        'span',
                        null,
                        'Search:'
                    ),
                    React.createElement(Input, null)
                ),
                info: ({ start, end, total, max, length }) => React.createElement(
                    Fragment,
                    null,
                    React.createElement(
                        Fragment,
                        null,
                        'Showing '
                    ),
                    start === 1 && length >= total ? React.createElement(
                        Fragment,
                        null,
                        'all'
                    ) : React.createElement(
                        Fragment,
                        null,
                        start,
                        ' to ',
                        end,
                        ' of'
                    ),
                    React.createElement(
                        Fragment,
                        null,
                        ' ',
                        total,
                        ' entries'
                    ),
                    total < max && React.createElement(
                        Fragment,
                        null,
                        ' (filtered from ',
                        max,
                        ' total entries)'
                    )
                ),
                infoLoading: "Showing … to … of … entries",
                infoEmpty: "Showing all 0 entries",
                loadingRecords: "Loading…",
                processing: ({ theme }) => React.createElement(
                    'div',
                    { className: theme.processing },
                    'Processing\u2026'
                ),
                zeroRecords: "No matching records found",
                emptyTable: "No data available in table",
                sortIcons: null
            },
            lengthMenu: [10, 25, 50, 100],
            searchDelay: 400,
            columns: [],
            start: 0,
            length: this.props.lengthMenu && this.props.lengthMenu.length ? this.props.lengthMenu[0] : 10,
            search: {
                value: '',
                regex: false
            },
            order: [[0, ASC]],
            rowComponent: DataTableRow,
            cellComponent: DataTableCell
        }),
              { api } = _defaultsDeep,
              options = objectWithoutProperties(_defaultsDeep, ['api']);
        if (options.order.length) {
            // https://datatables.net/reference/option/order
            const orderMap = {};
            for (let i = 0; i < options.columns.length; ++i) {
                if (isString(options.columns[i].name)) {
                    orderMap[options.columns[i].name] = i;
                } else if (isString(options.columns[i].data)) {
                    orderMap[options.columns[i].data] = i;
                }
            }
            if (!Array.isArray(options.order[0])) {
                options.order = [options.order];
            }
            options.order = options.order.map(([col, dir]) => {
                if (isString(col)) {
                    col = orderMap[col];
                }
                return [col, String(dir).toLowerCase()];
            });
        }

        return React.createElement(PureDataTable, _extends({ ref: this._ref }, options));
    }
}

// function fixOrder(order, map) {
//     if(!order) return;
//     if(Array.isArray(order[0])) {
//         for(let o of order) {
//             fixOrder(o, map);
//         }
//     } else if(hasProp(map, order[0])) {
//         order[0] = map[order[0]];
//     } else if(typeof order[0] === 'string') {
//         console.warn(`Column "${order[0]}" not found in order map`);
//         console.info(map);
//     }
// }

DataTable.propTypes = _extends({ api: PropTypes.func }, PureDataTable.propTypes);
function wheelTicks(ev) {
    switch (ev.deltaMode) {
        case MouseEvent.DOM_DELTA_PIXEL:
            return Math.ceil(ev.deltaY / 53);
        case MouseEvent.DOM_DELTA_LINE:
            return Math.ceil(ev.deltaY / 3);
        case MouseEvent.DOM_DELTA_PAGE:
            return Math.ceil(ev.deltaY);
    }
    return ev.deltaY < 0 ? -1 : 1;
}

export default DataTable;
//# sourceMappingURL=browser.js.map
